<?php

namespace Drupal\genpass\Plugin\Action;

use Drupal\Core\Action\ActionBase;
use Drupal\Core\Session\AccountInterface;

/**
 * Generate and set new random password for user and display it.
 *
 * @Action(
 *   id = "genpass_set_random_password",
 *   label = @Translation("Set new random password"),
 *   type = "user"
 * )
 */
class UserSetRandomPassword extends ActionBase {

  /**
   * {@inheritdoc}
   */
  public function execute($account = NULL) {
    if ($account !== FALSE && $account !== NULL) {
      // @see Drupal\user\Plugin\Action\BlockUser as to why original is clone.
      $account->original = clone $account;
      $password = genpass_generate();
      $account->setPassword($password);
      $account->save();

      $this->messenger()->addStatus($this->t(
        'Password for %account_name is: %new_password', [
          '%account_name' => $account->getDisplayName(),
          '%new_password' => $password,
        ]
      ));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function access($object, AccountInterface $account = NULL, $return_as_object = FALSE) {
    /** @var \Drupal\user\UserInterface $object */
    $access = $object->status->access('edit', $account, TRUE)
      ->andIf($object->access('update', $account, TRUE));

    return $return_as_object ? $access : $access->isAllowed();
  }

}
